@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -22,6 +23,7 @@
 
 #include "../Framework/Algorithms/ReconstructPyramidCommand.h"
 #include "../Framework/Algorithms/TranscodeTileCommand.h"
+#include "../Framework/ColorSpaces.h"
 #include "../Framework/DicomToolbox.h"
 #include "../Framework/DicomizerParameters.h"
 #include "../Framework/ImageToolbox.h"
@@ -29,6 +31,7 @@
 #include "../Framework/Inputs/CytomineImage.h"
 #include "../Framework/Inputs/HierarchicalTiff.h"
 #include "../Framework/Inputs/OpenSlidePyramid.h"
+#include "../Framework/Inputs/PlainTiff.h"
 #include "../Framework/Inputs/TiledJpegImage.h"
 #include "../Framework/Inputs/TiledPngImage.h"
 #include "../Framework/Inputs/TiledPyramidStatistics.h"
@@ -52,6 +55,8 @@
 #include <dcmtk/dcmdata/dcvrobow.h>
 #include <dcmtk/dcmdata/dcvrat.h>
 
+#include <boost/math/special_functions/round.hpp>
+
 
 static const char* OPTION_COLOR = "color";
 static const char* OPTION_COMPRESSION = "compression";
@@ -91,6 +96,10 @@ static const char* OPTION_CYTOMINE_PUBLIC_KEY = "cytomine-public-key";
 static const char* OPTION_CYTOMINE_PRIVATE_KEY = "cytomine-private-key";
 static const char* OPTION_CYTOMINE_COMPRESSION = "cytomine-compression";
 
+// New in release 2.1
+static const char* OPTION_FORCE_OPENSLIDE = "force-openslide";
+static const char* OPTION_PADDING = "padding";
+
 
 #if ORTHANC_FRAMEWORK_VERSION_IS_ABOVE(1, 9, 0)
 
@@ -557,6 +566,26 @@ static void EnrichDataset(DcmDataset& dataset,
   }
 
   SetupDimension(dataset, opticalPathId, source, volume);
+
+
+  // New in release 2.1
+  if (!dataset.tagExists(DCM_RecommendedAbsentPixelCIELabValue))
+  {
+    OrthancWSI::RGBColor rgb(parameters.GetBackgroundColorRed(),
+                             parameters.GetBackgroundColorGreen(),
+                             parameters.GetBackgroundColorBlue());
+    OrthancWSI::sRGBColor srgb(rgb);
+    OrthancWSI::XYZColor xyz(srgb);
+    OrthancWSI::LABColor lab(xyz);
+
+    uint16_t encoded[3];
+    lab.EncodeDicomRecommendedAbsentPixelCIELab(encoded);
+
+    if (!dataset.putAndInsertUint16Array(DCM_RecommendedAbsentPixelCIELabValue, encoded, 3).good())
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+    }
+  }
 }
 
 
@@ -575,6 +604,8 @@ static bool ParseParameters(int& exitStatus,
     (OPTION_THREADS,
      boost::program_options::value<int>()->default_value(parameters.GetThreadsCount()), 
      "Number of processing threads to be used")
+    (OPTION_FORCE_OPENSLIDE, boost::program_options::value<bool>()->default_value(false),
+     "Whether to force the use of OpenSlide on input TIFF-like files (Boolean)")
     (OPTION_OPENSLIDE, boost::program_options::value<std::string>(), 
      "Path to the shared library of OpenSlide "
      "(not necessary if converting from standard hierarchical TIFF)")
@@ -592,6 +623,9 @@ static bool ParseParameters(int& exitStatus,
      "Whether to repaint the background of the image (Boolean)")
     (OPTION_COLOR, boost::program_options::value<std::string>(),
      "Color of the background (e.g. \"255,0,0\")")
+    (OPTION_PADDING, boost::program_options::value<int>()->default_value(1),
+     "Add padding to plain PNG/JPEG/TIFF images to align the width/height to multiples "
+     "of this value, which enables deep zoom with IIIF (1 means no padding)")
     ;
 
   boost::program_options::options_description cytomine("Options if importing from Cytomine");
@@ -826,6 +860,12 @@ static bool ParseParameters(int& exitStatus,
     OrthancWSI::OpenSlideLibrary::Initialize(options[OPTION_OPENSLIDE].as<std::string>());
   }
 
+  if (options.count(OPTION_FORCE_OPENSLIDE) &&
+      options[OPTION_FORCE_OPENSLIDE].as<bool>())
+  {
+    parameters.SetForceOpenSlide(true);
+  }
+
   if (options.count(OPTION_PYRAMID) &&
       options[OPTION_PYRAMID].as<bool>())
   {
@@ -1004,6 +1044,20 @@ static bool ParseParameters(int& exitStatus,
     parameters.SetIccProfilePath(options[OPTION_ICC_PROFILE].as<std::string>());
   }
 
+  if (options.count(OPTION_PADDING))
+  {
+    int value = options[OPTION_PADDING].as<int>();
+    if (value <= 0)
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange,
+                                      "Padding must be >= 1");
+    }
+    else
+    {
+      parameters.SetPadding(static_cast<unsigned int>(value));
+    }
+  }
+
   return true;
 }
 
@@ -1033,26 +1087,36 @@ OrthancWSI::ITiledPyramid* OpenInputPyramid(OrthancWSI::ImageCompression& source
   OrthancWSI::ImageCompression format = OrthancWSI::DetectFormatFromFile(path);
   LOG(WARNING) << "File format of the input image: " << EnumerationToString(format);
 
+  std::unique_ptr<OrthancWSI::SingleLevelDecodedPyramid> plainImage;
+
   switch (format)
   {
     case OrthancWSI::ImageCompression_Png:
     {
       sourceCompression = OrthancWSI::ImageCompression_Unknown;
-      return new OrthancWSI::TiledPngImage(path, 
-                                           parameters.GetTargetTileWidth(512), 
-                                           parameters.GetTargetTileHeight(512));
+      plainImage.reset(new OrthancWSI::TiledPngImage(path,
+                                                     parameters.GetTargetTileWidth(512),
+                                                     parameters.GetTargetTileHeight(512)));
+      break;
     }
 
     case OrthancWSI::ImageCompression_Jpeg:
     {
       sourceCompression = OrthancWSI::ImageCompression_Unknown;
-      return new OrthancWSI::TiledJpegImage(path, 
-                                            parameters.GetTargetTileWidth(512), 
-                                            parameters.GetTargetTileHeight(512));
+      plainImage.reset(new OrthancWSI::TiledJpegImage(path,
+                                                      parameters.GetTargetTileWidth(512),
+                                                      parameters.GetTargetTileHeight(512)));
+      break;
     }
 
     case OrthancWSI::ImageCompression_Tiff:
     {
+      if (parameters.IsForceOpenSlide())
+      {
+        LOG(WARNING) << "Forcing the use of OpenSlide on a TIFF-like file";
+        break;
+      }
+
       try
       {
         std::unique_ptr<OrthancWSI::HierarchicalTiff> tiff(new OrthancWSI::HierarchicalTiff(path));
@@ -1061,7 +1125,19 @@ OrthancWSI::ITiledPyramid* OpenInputPyramid(OrthancWSI::ImageCompression& source
       }
       catch (Orthanc::OrthancException&)
       {
-        LOG(WARNING) << "This is not a standard hierarchical TIFF file";
+        LOG(WARNING) << "This is not a standard hierarchical TIFF file, fallback to plain TIFF";
+      }
+
+      try
+      {
+        sourceCompression = OrthancWSI::ImageCompression_Unknown;
+        plainImage.reset(new OrthancWSI::PlainTiff(path,
+                                                   parameters.GetTargetTileWidth(512),
+                                                   parameters.GetTargetTileHeight(512)));
+      }
+      catch (Orthanc::OrthancException&)
+      {
+        LOG(WARNING) << "This is not a standard plain TIFF file, fallback to OpenSlide (if enabled)";
       }
     }
 
@@ -1069,6 +1145,19 @@ OrthancWSI::ITiledPyramid* OpenInputPyramid(OrthancWSI::ImageCompression& source
       break;
   }
 
+  if (plainImage.get() != NULL)
+  {
+    if (parameters.GetPadding() > 1)
+    {
+      plainImage->SetPadding(parameters.GetPadding(),
+                             parameters.GetBackgroundColorRed(),
+                             parameters.GetBackgroundColorGreen(),
+                             parameters.GetBackgroundColorBlue());
+    }
+
+    return plainImage.release();
+  }
+
   try
   {
     LOG(WARNING) << "Trying to open the input pyramid with OpenSlide";
@@ -1078,6 +1167,10 @@ OrthancWSI::ITiledPyramid* OpenInputPyramid(OrthancWSI::ImageCompression& source
       new OrthancWSI::OpenSlidePyramid(path, parameters.GetTargetTileWidth(512),
                                        parameters.GetTargetTileHeight(512)));
 
+    openslide->SetBackgroundColor(parameters.GetBackgroundColorRed(),
+                                  parameters.GetBackgroundColorGreen(),
+                                  parameters.GetBackgroundColorBlue());
+
     float volumeWidth, volumeHeight;
     if (openslide->LookupImagedVolumeSize(volumeWidth, volumeHeight))
     {
@@ -1104,6 +1197,8 @@ OrthancWSI::ITiledPyramid* OpenInputPyramid(OrthancWSI::ImageCompression& source
 }
 
 
+#include "../Framework/ColorSpaces.h"
+
 int main(int argc, char* argv[])
 {
   OrthancWSI::ApplicationToolbox::GlobalInitialize();
