@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -73,7 +74,9 @@ namespace OrthancWSI
     opticalPath_(OpticalPath_Brightfield),
     isCytomineSource_(false),
     cytomineImageInstanceId_(-1),
-    cytomineCompression_(ImageCompression_Png)
+    cytomineCompression_(ImageCompression_Png),
+    forceOpenSlide_(false),
+    padding_(1)
   {
     backgroundColor_[0] = 255;
     backgroundColor_[1] = 255;
@@ -359,4 +362,17 @@ namespace OrthancWSI
       throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
     }
   }
+
+
+  void DicomizerParameters::SetPadding(unsigned int padding)
+  {
+    if (padding == 0)
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
+    }
+    else
+    {
+      padding_ = padding;
+    }
+  }
 }
