@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -67,6 +68,10 @@ namespace OrthancWSI
     std::string                    cytominePrivateKey_;
     ImageCompression               cytomineCompression_;
 
+    // New in release 2.1
+    bool          forceOpenSlide_;
+    unsigned int  padding_;
+
   public:
     DicomizerParameters();
 
@@ -278,5 +283,22 @@ namespace OrthancWSI
     int GetCytomineImageInstanceId() const;
 
     ImageCompression GetCytomineCompression() const;
+
+    void SetPadding(unsigned int padding);
+
+    unsigned int GetPadding() const
+    {
+      return padding_;
+    }
+
+    void SetForceOpenSlide(bool force)
+    {
+      forceOpenSlide_ = force;
+    }
+
+    bool IsForceOpenSlide() const
+    {
+      return forceOpenSlide_;
+    }
   };
 }
