@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -58,7 +59,8 @@ namespace OrthancWSI
   }
 
 
-  Orthanc::ImageAccessor* DecodedTiledPyramid::DecodeTile(unsigned int level,
+  Orthanc::ImageAccessor* DecodedTiledPyramid::DecodeTile(bool& isEmpty,
+                                                          unsigned int level,
                                                           unsigned int tileX,
                                                           unsigned int tileY)
   {
@@ -71,6 +73,7 @@ namespace OrthancWSI
     if (x >= GetLevelWidth(level) ||
         y >= GetLevelHeight(level))   // (*)
     {
+      isEmpty = true;
       ImageToolbox::Set(*tile, backgroundColor_[0], backgroundColor_[1], backgroundColor_[2]);
       return tile.release();
     }
@@ -103,14 +106,14 @@ namespace OrthancWSI
     if (fit)
     {
       // The tile entirely lies inside the image
-      ReadRegion(*tile, level, x, y);
+      ReadRegion(*tile, isEmpty, level, x, y);
     }
     else
     {
       // The tile exceeds the size of image, decode it to a temporary buffer
       std::unique_ptr<Orthanc::ImageAccessor> cropped
         (ImageToolbox::Allocate(GetPixelFormat(), regionWidth, regionHeight));
-      ReadRegion(*cropped, level, x, y);
+      ReadRegion(*cropped, isEmpty, level, x, y);
 
       // Create a white tile, and fill it with the cropped content
       ImageToolbox::Set(*tile, backgroundColor_[0], backgroundColor_[1], backgroundColor_[2]);
