@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -102,8 +103,15 @@ namespace OrthancWSI
             (tokens[1] != "THUMBNAIL" &&
              tokens[1] != "OVERVIEW"))
         {
+          if (instance->HasBackgroundColor())
+          {
+            backgroundRed_ = instance->GetBackgroundRed();
+            backgroundGreen_ = instance->GetBackgroundGreen();
+            backgroundBlue_ = instance->GetBackgroundBlue();
+          }
+
           instances_.push_back(instance.release());
-        }        
+        }
       }
       catch (Orthanc::OrthancException&)
       {
@@ -156,7 +164,10 @@ namespace OrthancWSI
                              const std::string& seriesId,
                              bool useCache) :
     orthanc_(orthanc),
-    seriesId_(seriesId)
+    seriesId_(seriesId),
+    backgroundRed_(255),
+    backgroundGreen_(255),
+    backgroundBlue_(255)
   {
     RegisterInstances(seriesId, useCache);
 
