@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -37,6 +38,9 @@ namespace OrthancWSI
     std::string                         seriesId_;
     std::vector<DicomPyramidInstance*>  instances_;
     std::vector<DicomPyramidLevel*>     levels_;
+    uint8_t                             backgroundRed_;
+    uint8_t                             backgroundGreen_;
+    uint8_t                             backgroundBlue_;
 
     void Clear();
 
@@ -84,5 +88,20 @@ namespace OrthancWSI
     virtual Orthanc::PixelFormat GetPixelFormat() const ORTHANC_OVERRIDE;
 
     virtual Orthanc::PhotometricInterpretation GetPhotometricInterpretation() const ORTHANC_OVERRIDE;
+
+    uint8_t GetBackgroundRed() const
+    {
+      return backgroundRed_;
+    }
+
+    uint8_t GetBackgroundGreen() const
+    {
+      return backgroundGreen_;
+    }
+
+    uint8_t GetBackgroundBlue() const
+    {
+      return backgroundBlue_;
+    }
   };
 }
