@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -46,6 +47,10 @@ namespace OrthancWSI
     std::vector<FrameLocation>          frames_;
     Orthanc::PhotometricInterpretation  photometric_;
     std::string                         imageType_;
+    bool                                hasBackgroundColor_;
+    uint8_t                             backgroundRed_;
+    uint8_t                             backgroundGreen_;
+    uint8_t                             backgroundBlue_;
 
     void Load(OrthancStone::IOrthancConnection&  orthanc,
               const std::string& instanceId);
@@ -109,5 +114,16 @@ namespace OrthancWSI
     unsigned int GetFrameLocationY(size_t frame) const;
 
     void Serialize(std::string& result) const;
+
+    bool HasBackgroundColor() const
+    {
+      return hasBackgroundColor_;
+    }
+
+    uint8_t GetBackgroundRed() const;
+
+    uint8_t GetBackgroundGreen() const;
+
+    uint8_t GetBackgroundBlue() const;
   };
 }
