@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -80,16 +81,6 @@ namespace OrthancWSI
   };
 
 
-  void HierarchicalTiff::Finalize()
-  {
-    if (tiff_)
-    {
-      TIFFClose(tiff_);
-      tiff_ = NULL;
-    }
-  }
-
-
   void HierarchicalTiff::CheckLevel(unsigned int level) const
   {
     if (level >= levels_.size())
@@ -99,85 +90,10 @@ namespace OrthancWSI
   }
 
 
-  bool HierarchicalTiff::GetCurrentCompression(ImageCompression& compression)
-  {
-    uint16_t c;
-    if (!TIFFGetField(tiff_, TIFFTAG_COMPRESSION, &c))
-    {
-      return false;
-    }
-
-    switch (c)
-    {
-      case COMPRESSION_NONE:
-        compression = ImageCompression_None;
-        return true;
-
-      case COMPRESSION_JPEG:
-        compression = ImageCompression_Jpeg;
-        return true;
-
-      default:
-        return false;
-    }
-  }
-
-
-  bool HierarchicalTiff::GetCurrentPixelFormat(Orthanc::PixelFormat& pixelFormat,
-                                               Orthanc::PhotometricInterpretation& photometric,
-                                               ImageCompression compression)
-  {
-    // http://www.awaresystems.be/imaging/tiff/tifftags/baseline.html
-
-    uint16_t channels, photometricTiff, bpp, planar;
-    if (!TIFFGetField(tiff_, TIFFTAG_SAMPLESPERPIXEL, &channels) ||
-        channels == 0 ||
-        !TIFFGetField(tiff_, TIFFTAG_PHOTOMETRIC, &photometricTiff) ||
-        !TIFFGetField(tiff_, TIFFTAG_BITSPERSAMPLE, &bpp) ||
-        !TIFFGetField(tiff_, TIFFTAG_PLANARCONFIG, &planar))
-    {
-      return false;
-    }
-
-    if (compression == ImageCompression_Jpeg &&
-        channels == 3 &&     // This is a color image
-        bpp == 8 &&
-        planar == PLANARCONFIG_CONTIG)  // This is interleaved RGB
-    {
-      pixelFormat = Orthanc::PixelFormat_RGB24;
-
-      switch (photometricTiff)
-      {
-        case PHOTOMETRIC_YCBCR:
-          photometric = Orthanc::PhotometricInterpretation_YBRFull422;
-          return true;
-          
-        case PHOTOMETRIC_RGB:
-          photometric = Orthanc::PhotometricInterpretation_RGB;
-          return true;
-
-        default:
-          LOG(ERROR) << "Unknown photometric interpretation in TIFF: " << photometricTiff;
-          return false;
-      }
-    }
-    else if (compression == ImageCompression_Jpeg &&
-             channels == 1 &&     // This is a grayscale image
-             bpp == 8)
-    {
-      pixelFormat = Orthanc::PixelFormat_Grayscale8;
-      photometric = Orthanc::PhotometricInterpretation_Monochrome2;
-    }
-    else
-    {
-      return false;
-    }
-
-    return true;
-  }
-
-
-  bool  HierarchicalTiff::Initialize()
+  HierarchicalTiff::HierarchicalTiff(const std::string& path) :
+    reader_(path),
+    tileWidth_(0),
+    tileHeight_(0)
   {
     bool first = true;
     tdir_t pos = 0;
@@ -189,17 +105,16 @@ namespace OrthancWSI
       Orthanc::PixelFormat pixelFormat;
       Orthanc::PhotometricInterpretation photometric;
 
-      if (TIFFSetDirectory(tiff_, pos) &&
-          TIFFGetField(tiff_, TIFFTAG_IMAGEWIDTH, &w) &&
-          TIFFGetField(tiff_, TIFFTAG_IMAGELENGTH, &h) &&
-          TIFFGetField(tiff_, TIFFTAG_TILEWIDTH, &tw) &&
-          TIFFGetField(tiff_, TIFFTAG_TILELENGTH, &th) &&
+      if (TIFFSetDirectory(reader_.GetTiff(), pos) &&
+          TIFFGetField(reader_.GetTiff(), TIFFTAG_IMAGEWIDTH, &w) &&
+          TIFFGetField(reader_.GetTiff(), TIFFTAG_IMAGELENGTH, &h) &&
+          TIFFGetField(reader_.GetTiff(), TIFFTAG_TILEWIDTH, &tw) &&
+          TIFFGetField(reader_.GetTiff(), TIFFTAG_TILELENGTH, &th) &&
           w > 0 &&
           h > 0 &&
           tw > 0 &&
           th > 0 &&
-          GetCurrentCompression(compression) &&
-          GetCurrentPixelFormat(pixelFormat, photometric, compression))
+          reader_.GetCurrentDirectoryInformation(compression, pixelFormat, photometric))
       {
         if (first)
         {
@@ -216,44 +131,23 @@ namespace OrthancWSI
                  pixelFormat_ != pixelFormat ||
                  photometric_ != photometric)
         {
-          LOG(ERROR) << "The tile size or compression of the TIFF file varies along levels, this is not supported";
-          return false;
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat,
+                                          "The tile size or compression of the TIFF file varies along levels, this is not supported");
         }
 
-        levels_.push_back(Level(tiff_, pos, w, h));
+        levels_.push_back(Level(reader_.GetTiff(), pos, w, h));
       }
 
       pos++;
     }
-    while (TIFFReadDirectory(tiff_));
+    while (TIFFReadDirectory(reader_.GetTiff()));
 
     if (levels_.size() == 0)
     {
-      LOG(ERROR) << "This is not a tiled TIFF image";
-      return false;
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat, "This is not a tiled TIFF image");
     }
 
     std::sort(levels_.begin(), levels_.end(), Comparator());
-    return true;
-  }
-
-
-  HierarchicalTiff::HierarchicalTiff(const std::string& path) :
-    tileWidth_(0),
-    tileHeight_(0)
-  {
-    tiff_ = TIFFOpen(path.c_str(), "r");
-    if (tiff_ == NULL)
-    {
-      LOG(ERROR) << "libtiff cannot open: " << path;
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_InexistentFile);
-    }
-
-    if (!Initialize())
-    {
-      Finalize();
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
-    }
   }
 
 
@@ -284,17 +178,17 @@ namespace OrthancWSI
     compression = compression_;
 
     // Make the TIFF context point to the level of interest
-    if (!TIFFSetDirectory(tiff_, levels_[level].directory_))
+    if (!TIFFSetDirectory(reader_.GetTiff(), levels_[level].directory_))
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_CorruptedFile);
     }
 
     // Get the index of the tile
-    ttile_t index = TIFFComputeTile(tiff_, tileX * tileWidth_, tileY * tileHeight_, 0 /*z*/, 0 /*sample*/);
+    ttile_t index = TIFFComputeTile(reader_.GetTiff(), tileX * tileWidth_, tileY * tileHeight_, 0 /*z*/, 0 /*sample*/);
 
     // Read the raw tile
     toff_t *sizes;
-    if (!TIFFGetField(tiff_, TIFFTAG_TILEBYTECOUNTS, &sizes))
+    if (!TIFFGetField(reader_.GetTiff(), TIFFTAG_TILEBYTECOUNTS, &sizes))
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_CorruptedFile);
     }
@@ -302,7 +196,7 @@ namespace OrthancWSI
     std::string raw;
     raw.resize(sizes[index]);
 
-    tsize_t read = TIFFReadRawTile(tiff_, index, &raw[0], raw.size());
+    tsize_t read = TIFFReadRawTile(reader_.GetTiff(), index, &raw[0], raw.size());
     if (read != static_cast<tsize_t>(sizes[index]))
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_CorruptedFile);
