@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -29,7 +30,8 @@ namespace OrthancWSI
   class PyramidWithRawTiles : public ITiledPyramid
   {
   public:
-    virtual Orthanc::ImageAccessor* DecodeTile(unsigned int level,
+    virtual Orthanc::ImageAccessor* DecodeTile(bool& isEmpty,
+                                               unsigned int level,
                                                unsigned int tileX,
                                                unsigned int tileY) ORTHANC_OVERRIDE;
   };
