@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -106,10 +107,13 @@ namespace OrthancWSI
   }
 
 
-  Orthanc::ImageAccessor* InMemoryTiledImage::DecodeTile(unsigned int level,
+  Orthanc::ImageAccessor* InMemoryTiledImage::DecodeTile(bool& isEmpty,
+                                                         unsigned int level,
                                                          unsigned int tileX,
                                                          unsigned int tileY)
   {
+    isEmpty = false;
+
     CheckLevel(level);
 
     if (tileX >= countTilesX_ ||
